package org.apache.bcel.generic;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.io.DataOutputStream;
import java.io.IOException;
import org.apache.bcel.ExceptionConst;
import org.apache.bcel.classfile.Constant;
import org.apache.bcel.classfile.ConstantClass;
import org.apache.bcel.classfile.ConstantFloat;
import org.apache.bcel.classfile.ConstantInteger;
import org.apache.bcel.classfile.ConstantString;
import org.apache.bcel.classfile.ConstantUtf8;
import org.apache.bcel.generic.CPInstruction;
import org.apache.bcel.generic.ConstantPoolGen;
import org.apache.bcel.generic.ExceptionThrower;
import org.apache.bcel.generic.ObjectType;
import org.apache.bcel.generic.PushInstruction;
import org.apache.bcel.generic.Type;
import org.apache.bcel.generic.Visitor;
import org.apache.bcel.util.ByteSequence;

public class LDC extends CPInstruction implements PushInstruction, ExceptionThrower {
  LDC() {}
  
  public LDC(int index) {
    super((short)19, index);
    setSize();
  }
  
  public void accept(Visitor v) {
    v.visitStackProducer(this);
    v.visitPushInstruction(this);
    v.visitExceptionThrower(this);
    v.visitTypedInstruction(this);
    v.visitCPInstruction(this);
    v.visitLDC(this);
  }
  
  public void dump(DataOutputStream out) throws IOException {
    out.writeByte(getOpcode());
    if (getLength() == 2) {
      out.writeByte(getIndex());
    } else {
      out.writeShort(getIndex());
    } 
  }
  
  public Class<?>[] getExceptions() {
    return ExceptionConst.createExceptions(ExceptionConst.EXCS.EXCS_STRING_RESOLUTION, new Class[0]);
  }
  
  public Type getType(ConstantPoolGen cpg) {
    switch (cpg.getConstantPool().getConstant(getIndex()).getTag()) {
      case 8:
        return Type.STRING;
      case 4:
        return Type.FLOAT;
      case 3:
        return Type.INT;
      case 7:
        return Type.CLASS;
    } 
    throw new IllegalArgumentException("Unknown or invalid constant type at " + getIndex());
  }
  
  public Object getValue(ConstantPoolGen cpg) {
    int i, nameIndex;
    Constant c = cpg.getConstantPool().getConstant(getIndex());
    switch (c.getTag()) {
      case 8:
        i = ((ConstantString)c).getStringIndex();
        c = cpg.getConstantPool().getConstant(i);
        return ((ConstantUtf8)c).getBytes();
      case 4:
        return Float.valueOf(((ConstantFloat)c).getBytes());
      case 3:
        return Integer.valueOf(((ConstantInteger)c).getBytes());
      case 7:
        nameIndex = ((ConstantClass)c).getNameIndex();
        c = cpg.getConstantPool().getConstant(nameIndex);
        return new ObjectType(((ConstantUtf8)c).getBytes());
    } 
    throw new IllegalArgumentException("Unknown or invalid constant type at " + getIndex());
  }
  
  protected void initFromFile(ByteSequence bytes, boolean wide) throws IOException {
    setLength(2);
    super.setIndex(bytes.readUnsignedByte());
  }
  
  public final void setIndex(int index) {
    super.setIndex(index);
    setSize();
  }
  
  protected final void setSize() {
    if (getIndex() <= 255) {
      setOpcode((short)18);
      setLength(2);
    } else {
      setOpcode((short)19);
      setLength(3);
    } 
  }
}
